@@ -1,12 +1,12 @@
 TFRnewPred.group <- function(g, main.win, parent) {
-	nb <- gnotebook(container=g, expand=TRUE)
-	all.c.g <- ggroup(label="<span color='#0B6138'>All Countries</span>", 
+	nb <- bDem.gnotebook(container=g, expand=TRUE)
+	all.c.g <- ggroup(label="  <span color='#0B6138'>All Countries</span>  ", 
 							markup=TRUE, horizontal=FALSE, expand=TRUE, container=nb)
 	pred.all <- tfr.pred.all.countries.group(all.c.g, main.win, parent)
-	extra.c.g <- ggroup(label="<span color='#0B6138'>Extra Areas &amp; Regions</span>", 
+	extra.c.g <- ggroup(label="  <span color='#0B6138'>Extra Areas &amp; Regions</span>  ", 
 							markup=TRUE, horizontal=FALSE, expand=TRUE, container=nb)
 	pred.extra <- tfr.pred.extra.countries.group(extra.c.g, main.win, parent)
-	edit.g <- ggroup(label="<span color='#0B6138'>Edit Predictions</span>", 
+	edit.g <- ggroup(label="  <span color='#0B6138'>Edit Predictions</span>  ", 
 							markup=TRUE, horizontal=FALSE, expand=TRUE, container=nb)
 	pred.edit <- edit.predictions.group(edit.g, main.win, parent)
 	svalue(nb) <- 1
@@ -14,70 +14,70 @@ TFRnewPred.group <- function(g, main.win, parent) {
 }
 
 tfr.pred.all.countries.group <- function(g, main.win, parent) {
-	enable.pred.settings <- function(not.use.diag) {
-		enabled(e$burnin) <- not.use.diag
-		enabled(e$nr.traj) <- not.use.diag
-		enabled(e$thin) <- not.use.diag	
-	}
 	e <- new.env()
 	defaults <- formals(tfr.predict) # default argument values
 	e$sim.dir <- parent$sim.dir
-	
-	pred.g <- gframe("<span color='blue'>Prediction</span>", markup=TRUE, horizontal=FALSE, container=g)
-	pred.g1 <- ggroup(horizontal=TRUE, container=pred.g)
-	glabel("End year:", container=pred.g1)
-	glabel("<span color='red'>*</span>", markup=TRUE, container=pred.g1)
-	e$end.year <- gedit(defaults$end.year, width=4, container=pred.g1)
-	e$use.diagnostics <- gcheckbox("Use diagnostics", checked = defaults$use.diagnostics, 
-									handler=function(h, ...) enable.pred.settings(!svalue(h$obj)), 
-									container=pred.g1)
-	addSpace(pred.g1, 20)
-	glabel("RNG seed:", container=pred.g1)
-	e$seed <- gedit(defaults$seed, width=4, container=pred.g1)
-	addSpace(pred.g1, 10)
-	e$verbose <- gcheckbox("Verbose", checked=defaults$verbose, container=pred.g1)
-
-	pred.g2 <- ggroup(horizontal=TRUE, container=pred.g)
-	glabel("Burnin:", container=pred.g2)
-	glabel("<span color='red'>*</span>", markup=TRUE, container=pred.g2)
-	e$burnin <- gedit(defaults$burnin, width=7, container=pred.g2)
-	glabel("Nr. of trajectories:", container=pred.g2)
-	e$nr.traj <- gedit(defaults$nr.traj, width=5, container=pred.g2)
-	glabel("OR  Thin:", container=pred.g2)
-	e$thin <- gedit(defaults$thin, width=5, container=pred.g2)
-	enable.pred.settings(!defaults$use.diagnostics)
-	
-	sim.g <- gframe("<span color='blue'>Output settings</span>", markup=TRUE, horizontal=FALSE, container=g)
-	out.g1 <- ggroup(horizontal=TRUE, container=sim.g)
-	e$replace.output <- gcheckbox("Overwrite existing prediction in simulation directory", 
-									checked=defaults$replace.output, container=out.g1)
-	sim.g2 <- ggroup(horizontal=TRUE, container=sim.g)
-
-	glabel("Nr. of ascii trajectories:", container=sim.g2)
-	e$save.as.ascii <- gedit(defaults$save.as.ascii, width=5, container=sim.g2)
-	
+	addSpace(g, 10)
+	.create.prediction.setting.group(g, e, defaults)
+	addSpace(g, 10)
 	ar.g <- gframe("<span color='blue'>AR(1) Process</span>", markup=TRUE, horizontal=TRUE, container=g)
 	glabel("mu:", container=ar.g)
-	e$mu <- gedit(defaults$mu, width=5, container=ar.g)
+	e$mu <- gedit(defaults$mu, width=10, container=ar.g)
+	addSpace(ar.g, 10)
 	glabel("rho:", container=ar.g)
-	e$rho <- gedit(defaults$rho, width=6, container=ar.g)
+	e$rho <- gedit(defaults$rho, width=10, container=ar.g)
+	addSpace(ar.g, 10)
 	glabel("sigma:", container=ar.g)
-	e$sigmaAR1 <- gedit(defaults$sigmaAR1, width=15, container=ar.g)
-	
+	e$sigmaAR1 <- gedit(defaults$sigmaAR1, width=10, container=ar.g)
+	addSpace(g, 10)
+	.create.status.label(g, e)
 	addSpring(g)
 	predict.g <- ggroup(horizontal=TRUE, container=g)
 	create.help.button(topic='tfr.predict', package='bayesTFR', 
 				parent.group=predict.g,
 						parent.window=main.win)
 	addSpring(predict.g)
-	gbutton(' Generate Script ', container=predict.g, handler=run.tfr.prediction,
-				action=list(mw=main.win, env=e, script=TRUE))
-	gbutton(action=gaction(label=' Make Prediction ', icon='evaluate', handler=run.tfr.prediction, 
+	create.generate.script.button(handler=run.tfr.prediction, action=list(mw=main.win, env=e, script=TRUE),
+								container=predict.g)
+	bDem.gbutton(action=gaction(label=' Make Prediction ', icon='evaluate', handler=run.tfr.prediction, 
 				action=list(mw=main.win, env=e, script=FALSE)), container=predict.g)
 	return(e)
 
 }
 
+.create.prediction.setting.group <- function(g, e, defaults) {
+	enable.pred.settings <- function(not.use.diag) {
+		enabled(e$burnin) <- not.use.diag
+		enabled(e$nr.traj) <- not.use.diag
+		enabled(e$thin) <- not.use.diag	
+	}
+	leftcenter <- c(-1,0)
+	pred.g <- gframe("<span color='blue'>Prediction settings</span>", markup=TRUE, horizontal=FALSE, container=g)
+	#pred.g1 <- ggroup(horizontal=TRUE, container=pred.g)
+	plo <- glayout(container=pred.g)
+	plo[1,1, anchor=leftcenter] <- glabel("End year:", container=plo)
+	#glabel("<span color='red'>*</span>", markup=TRUE, container=pred.g1)
+	plo[1,2] <- e$end.year <- gedit(defaults$end.year, width=4, container=plo)
+	plo[1,4, anchor=leftcenter] <- 	glabel("Burnin:", container=plo)
+	#glabel("<span color='red'>*</span>", markup=TRUE, container=pred.g2)
+	plo[1,5] <- e$burnin <- gedit(defaults$burnin, width=7, container=plo)
+	plo[2,1, anchor=leftcenter] <- glabel("Nr. trajectories:", container=plo)
+	plo[2,2] <- e$nr.traj <- gedit(defaults$nr.traj, width=5, container=plo)
+	plo[2,3, anchor=leftcenter] <- 'OR'
+	plo[2,4, anchor=leftcenter] <- glabel("Thin:", container=plo)
+	plo[2,5] <- e$thin <- gedit(defaults$thin, width=5, container=plo)
+	plo[3,1, anchor=leftcenter] <- glabel("Nr. ascii trajectories:", container=plo)
+	plo[3,2] <- e$save.as.ascii <- gedit(defaults$save.as.ascii, width=5, container=plo)
+	plo[3,4, anchor=leftcenter] <- glabel("RNG seed:", container=plo)
+	plo[3,5] <- e$seed <- gedit(defaults$seed, width=4, container=plo)
+	plo[2,6] <- e$use.diagnostics <- gcheckbox("Use diagnostics", checked = defaults$use.diagnostics, 
+									handler=function(h, ...) enable.pred.settings(!svalue(h$obj)), 
+									container=plo)
+	plo[3,6] <- e$verbose <- gcheckbox("Verbose", checked=defaults$verbose, container=plo)
+	enable.pred.settings(!defaults$use.diagnostics)
+	return(plo)
+}
+
 run.tfr.prediction <- function(h, ...)
 {
 	e <- h$action$env
@@ -86,7 +86,7 @@ run.tfr.prediction <- function(h, ...)
 	param.names <- list(numeric=c('mu', 'rho', 'end.year', 'burnin', 'seed', 'nr.traj', 'thin'), 
 						numvector=c('sigmaAR1'),
 						text=c('sim.dir'),
-						logical=c('verbose', 'replace.output', 'use.diagnostics'),
+						logical=c('verbose', 'use.diagnostics'),
 						numtext=c('save.as.ascii') #can be both - numeric or text
 						)
 	params <- get.parameters(param.names, e, h$action$script)
@@ -97,17 +97,21 @@ run.tfr.prediction <- function(h, ...)
 		params[['thin']] <- NULL
 		params[['nr.traj']] <- NULL
 	}
+	
+	simdir <- get.parameters(list(text='sim.dir'), e, quote=FALSE)$sim.dir # to avoid double quoting if script is TRUE
+	if(has.tfr.prediction(sim.dir=simdir)) {
+		params[['replace.output']] <- FALSE
+		if (gconfirm(paste('Prediction for', simdir, 
+								'already exists.\nDo you want to overwrite existing results?'),
+				icon='question', parent=h$action$mw))
+			params[['replace.output']] <- TRUE
+		else return(NULL)
+	}
+
 	if (h$action$script) {
-		script.text <- gwindow('bayesTFR commands', parent=h$action$mw)
-		gtext(paste('tfr.predict(', paste(paste(names(params), params, sep='='), collapse=', '),
-											 ')',sep=' '), 
-					container=script.text)
+		cmd <- paste('tfr.predict(', assemble.arguments(params), ')', sep=' ')
+		create.script.widget(cmd, h$action$mw, package="bayesTFR")
 	} else {
-		if(!params[['replace.output']] & has.tfr.prediction(sim.dir=params[['sim.dir']])) {
-				gmessage(paste('Prediction for', params[['sim.dir']], 
-								'already exists.\nCheck "Overwrite existing prediction" to delete it.'))
-				return()
-		}
 		if(params$use.diagnostics) {
 			mcmc.set <- get.tfr.mcmc(params[['sim.dir']])
 			diag.list <- get.tfr.convergence.all(mcmc.set$meta$output.dir)
@@ -117,46 +121,77 @@ run.tfr.prediction <- function(h, ...)
 				return()
 			}
 		}
-		do.call('tfr.predict', params)
+		.run.prediction(e, type='TFR', handler=get.tfr.prediction.status, option='bDem.TFRpred', 
+								call='tfr.predict', params=params, 
+								sim.name='TFR prediction', main.win=h$action$mw,
+								action=list(sb=e$statuslabel),
+								interval=1000)
 	}
 }
+
+.run.prediction <- function(type='TFR', ...) {
+	statusopt <- paste('bDem.', type, 'pred.status', sep='')
+	opt <- list()
+	opt[statusopt] <- list(NULL)
+	options(opt)
+	res <- .run.simulation(...)
+	options(opt)
+	return(res)
+}
+.update.status <- function(sb, status.option, prefix) {
+	predstatus <- getOption(status.option, default=NULL)
+	status <- prefix
+	if(!is.null(predstatus))
+		status <- paste(status, predstatus)
+	svalue(sb) <- status
+}
+
+get.tfr.prediction.status <- function(h, ...) 
+	.update.status(h$action$sb, 'bDem.TFRpred.status', 'Running TFR prediction ...')
+
+
+.create.extra.countries.group <- function(g, e, main.win, defaults) {
+	addSpace(g, 10)
+	countries.g <- gframe("<span color='blue'>Countries/Regions selection</span>", markup=TRUE, 
+							horizontal=FALSE, container=g)
+	countries.g1 <- glayout(container=countries.g)
+	countries.g1[1,1] <- e$all.countries <- gcheckbox("All without prediction", checked=TRUE, container=countries.g1,
+									handler=function(h,...){
+										enabled(e$e.countries.gb) <- !svalue(h$obj)
+										})
+	countries.g1[1,2] <- '      '
+	countries.g1[1,3] <- e$e.countries.gb <- bDem.gbutton("  Select specific countries/regions  ", container=countries.g1,
+				handler=selectCountryMenuPred,
+				action=list(mw=main.win, env=e, not.predicted=TRUE, multiple=TRUE, sorted=FALSE, 
+							type=e$pred.type, label.widget.name='extra.pred.country.label'))
+	enabled(e$e.countries.gb) <- !svalue(e$all.countries)
+	countries.g1[2,3, anchor=c(-1,0)] <- e$extra.pred.country.label <- glabel('', container=countries.g1)
+	addSpace(g, 10)
+	sim.g <- gframe("<span color='blue'>Output</span>", markup=TRUE, horizontal=FALSE, container=g)
+	sim.g2 <- ggroup(horizontal=TRUE, container=sim.g)
+	glabel("Nr. of ascii trajectories:", container=sim.g2)
+	e$save.as.ascii <- gedit(defaults$save.as.ascii, width=5, container=sim.g2)
+	addSpace(sim.g2, 30)
+	e$verbose <- gcheckbox("Verbose", checked=defaults$verbose, container=sim.g2)
+	addSpace(g, 10)
+	.create.status.label(g, e)
+}
 	
 tfr.pred.extra.countries.group <- function(g, main.win, parent) {
 	e <- new.env()
 	defaults <- formals(tfr.predict.extra) # default argument values
-		
-	e$countries.g <- gframe("<span color='blue'>Countries/Regions selection</span>", markup=TRUE, 
-							horizontal=FALSE, container=g)
-	e$countries.g1 <- ggroup(horizontal=TRUE, container=e$countries.g)
-	e$all.countries <- gcheckbox("All without prediction", checked=TRUE, container=e$countries.g1,
-									handler=function(h,...){
-										enabled(e$e.countries.gb) <- !svalue(h$obj)
-										})
-	addSpace(e$countries.g1, 20)
-	e$e.countries.gb <- gbutton("  Select specific countries/regions  ", container=e$countries.g1,
-				handler=selectCountryMenuPred,
-				action=list(mw=main.win, env=e, not.predicted=TRUE, multiple=TRUE, sorted=FALSE))
-	enabled(e$e.countries.gb) <- !svalue(e$all.countries)
-	
-	e$sim.g <- gframe("<span color='blue'>Output</span>", markup=TRUE, horizontal=FALSE, container=g)
 	e$sim.dir <- parent$sim.dir
-	e$sim.g2 <- ggroup(horizontal=TRUE, container=e$sim.g)
-	glabel("# ascii trajectories:", container=e$sim.g2)
-	e$save.as.ascii <- gedit(defaults$save.as.ascii, width=5, container=e$sim.g2)
-	addSpace(e$sim.g2, 30)
-	e$verbose <- gcheckbox("Verbose", checked=defaults$verbose, container=e$sim.g2)
-
+	e$pred.type <- 'tfr'
+	.create.extra.countries.group(g, e, main.win, defaults)
 	addSpring(g)
 	predict.g <- ggroup(horizontal=TRUE, container=g)
 	create.help.button(topic='tfr.predict.extra', package='bayesTFR', 
-				parent.group=predict.g,
-						parent.window=main.win)
+				parent.group=predict.g, parent.window=main.win)
 	addSpring(predict.g)
-	gbutton(' Generate Script ', container=predict.g, handler=run.tfr.prediction.extra,
-				action=list(mw=main.win, env=e, script=TRUE))
-	gbutton(action=gaction(label=' Make Prediction ', icon='evaluate', handler=run.tfr.prediction.extra, 
+	create.generate.script.button(handler=run.tfr.prediction.extra, action=list(mw=main.win, env=e, script=TRUE),
+								container=predict.g)
+	bDem.gbutton(action=gaction(label=' Make Prediction ', icon='evaluate', handler=run.tfr.prediction.extra, 
 				action=list(mw=main.win, env=e, script=FALSE)), container=predict.g)
-
 	return(e)
 		  
 }
@@ -173,32 +208,33 @@ run.tfr.prediction.extra <- function(h, ...)
 	params[['countries']] <- if(svalue(e$all.countries)) NULL else e$selected.countries
 	
 	if (h$action$script) {
-		script.text <- gwindow('bayesTFR commands', parent=h$action$mw)
-		gtext(paste('tfr.predict.extra(', paste(paste(names(params), params, sep='='), collapse=', '),
-											 ')',sep=' '), 
-					container=script.text)
+		cmd <- paste('tfr.predict.extra(', assemble.arguments(params), ')',sep=' ')
+		create.script.widget(cmd, h$action$mw, package="bayesTFR")
 	} else {
-		do.call('tfr.predict.extra', params)
+		.run.prediction(e, type='TFR', handler=get.tfr.prediction.status, option='bDem.TFRpred', 
+								call='tfr.predict.extra', params=params, 
+								sim.name='TFR extra prediction', main.win=h$action$mw,
+								action=list(sb=e$statuslabel), interval=1000)
 	}
 }
 
 edit.predictions.group <- function(g, main.win, parent, type='tfr') {
 	e <- new.env()
 	e$sim.dir <- parent$sim.dir
+	addSpace(g, 10)
 	e$edit.frame <- gframe("<span color='blue'>Edit Median</span>", markup=TRUE, 
 						horizontal=FALSE, container=g, expand=TRUE)
-	gbutton("  Select specific country/region  ", container=e$edit.frame,
+	bDem.gbutton("  Select specific country/region  ", container=e$edit.frame,
 				handler=selectCountryMenuPred,
 				action=list(mw=main.win, env=e, not.predicted=FALSE, multiple=FALSE,
 							edit.median=TRUE, sorted=TRUE, type=type))
-
 	#addSpring(g)
 	button.g <- ggroup(horizontal=TRUE, container=g)
 	addSpring(button.g)
-	e$restoreb <- gbutton(action=gaction(label='Restore BHM medians', 
+	e$restoreb <- bDem.gbutton(action=gaction(label='Restore BHM medians', 
 				handler=restore.bhm.medians, action=list(mw=main.win, env=e, type=type)), container=button.g)
 	enabled(e$restoreb) <- FALSE
-	e$applyb <- gbutton(action=gaction(label='Save', handler=edit.prediction, 
+	e$applyb <- bDem.gbutton(action=gaction(label='Save', handler=edit.prediction, 
 				action=list(mw=main.win, env=e, type=type)), container=button.g)
 	enabled(e$applyb) <- FALSE
 	
@@ -210,7 +246,7 @@ edit.predictions.group <- function(g, main.win, parent, type='tfr') {
 	e$write.summary <- gcheckbox("Write summary files", checked=TRUE, container=convert.frame)
 	e$verbose <- gcheckbox("Verbose", checked=FALSE, container=convert.frame)
 	addSpring(convert.frame)
-	button.conv <- gbutton(action=gaction(label='Convert', handler=.convert.trajectories, 
+	bDem.gbutton(action=gaction(label='Convert', handler=.convert.trajectories, 
 				action=list(mw=main.win, env=e, type=type)), container=convert.frame)
 }
 
@@ -312,6 +348,8 @@ selectCountryMenuPred <- function(h, ...) {
 		h$action$env$selected.countries <- svalue(h$action$env$sel.extra.country.gt)
 		if(!is.null(h$action$edit.median) && h$action$edit.median) show.median.editor(h$action$env, h$action$type) 
 		visible(h$action$env$extra.country.sel.win) <- FALSE
+		if(!is.null(h$action$label.widget.name) && !is.null(h$action$env[[h$action$label.widget.name]])) 
+			svalue(h$action$env[[h$action$label.widget.name]]) <- paste(h$action$env$selected.countries, collapse=',')
 	}
 	new.window <- TRUE
 	if (!is.null(h$action$env$extra.country.sel.win)) {
@@ -361,5 +399,4 @@ selectCountryMenuPred <- function(h, ...) {
 		removehandler(h$action$env$sel.extra.country.okbutton, h$action$env$sel.extra.country.ok.handler)
 	h$action$env$sel.extra.country.ok.handler <- addhandlerclicked(
 						h$action$env$sel.extra.country.okbutton, handler=country.selected)
-
 }
